<?php

namespace SIU\SQ\Lib\daos;

use SIU\SQ\Lib\constantes;
use SIU\SQ\Lib\errores\error_negocio;

class dao_comprobantes extends dao
{
    public function get_lista($filtro = [], $order_by = "ORDER BY id", $limit = "")
    {
        $where = "";
        if (isset($filtro['id'])) {
            $where .= " AND id = " . $this->quote($filtro['id']);
        }
        if (isset($filtro['id_comprobante_template'])) {
            $where .= " AND id_comprobante_template = " . $this->quote($filtro['id_comprobante_template']);
        }
        if (isset($filtro['id_punto_cobranza'])) {
            $where .= " AND id_punto_cobranza = " . $this->quote($filtro['id_punto_cobranza']);
        }
        if (isset($filtro['estado'])) {
            $where .= " AND estado = " . $this->quote($filtro['estado']);
        }
        if (isset($filtro['numero_comprobante'])) {
            $where .= " AND numero_comprobante = " . $this->quote($filtro['numero_comprobante']);
        }

        $venta_info_personalizada = "(SELECT vip.datos FROM ventas_info_personalizada vip
       				                  JOIN cobranzas_ventas cv ON cv.id_venta = vip.id_venta
       				                  JOIN cobranzas c ON cv.id_cobranza = c.id AND c.id_comprobante = comp.id
        	            ) as ventas_info_personalizada";

        if (isset($filtro['id_comprobante_relacionado'])) {
            $where .= " AND id_comprobante_relacionado = " . $this->quote($filtro['id_comprobante_relacionado']);

            $venta_info_personalizada = "(SELECT vip.datos FROM ventas_info_personalizada vip
       				                  JOIN cobranzas_ventas cv ON cv.id_venta = vip.id_venta
       				                  JOIN cobranzas c ON cv.id_cobranza = c.id AND c.id_comprobante = comp.id_comprobante_relacionado
                	    ) as ventas_info_personalizada";
        }

        $sql = "SELECT
					comp.id,
					comp.id_comprobante_template,
					comp.presentacion,
       				comp.importe_control,
					comp.id_rdi,
					comp.uuid_arai_documentos,
					comp.numero_comprobante,
       				comp.id_comprobante_relacionado,
       				comp.estado,
       				comp.estado_actualizado_en,
       				comp.id_punto_cobranza,
                    $venta_info_personalizada
				FROM
					comprobantes comp
				WHERE 1=1
					$where $order_by $limit;";
        return $this->entorno()->db()->consultar($sql);
    }

    public function get($id)
    {
        $rs = $this->get_lista(['id' => $id]);
        if (count($rs) == 1) {
            $comprobante = $rs[0];
            $comprobante['presentacion'] = utf8_e_seguro($comprobante['presentacion']);
            return $comprobante;
        } else {
            if (count($rs) > 1) {
                throw new error_negocio("DAO_COMPROBANTES: "
                        . "La consulta devuelve mas de un registro.");
            } else {
                return null;
            }
        }
    }

    public function get_lista_templates($filtro = [], $order_by = "ORDER BY ct.id", $limit = "")
    {
        $where = "";
        if (isset($filtro['id'])) {
            $where .= " AND ct.id = " . $this->quote($filtro['id']);
        }
        if (isset($filtro['codigo'])) {
            $where .= " AND ct.codigo = " . $this->quote($filtro['codigo']);
        }
        if (isset($filtro['descripcion'])) {
            $where .= " AND ct.descripcion = " . $this->quote($filtro['descripcion']);
        }
        if (isset($filtro['estado'])) {
            $where .= " AND ct.estado = " . $this->quote($filtro['estado']);
        }

        $sql = "SELECT
					ct.id,
					ct.codigo,
					ct.descripcion,
					ct.tipo_comprobante,
					ct.modelo,
					ct.numeracion,
					ct.estado,
					ct.estado_actualizado_en,
					ct.id_arai,
					ct.revision,
					ct.sincronizar,
                    true as numeracion_externa
				FROM
					comprobantes_templates ct
				WHERE 1 = 1
					$where
				$order_by $limit";
        return $this->entorno()->db()->consultar($sql);
    }

    public function get_template($id)
    {
        $rs = $this->get_lista_templates(['id' => $id]);
        if (count($rs) == 1) {
            return $rs[0];
        } else {
            if (count($rs) > 1) {
                throw new error_negocio("DAO_COMPROBANTES: "
                        . "La consulta devuelve mas de un registro.");
            } else {
                return null;
            }
        }
    }

    public function get_parametrizaciones()
    {
        $sql = "
            SELECT
                p.id,  
                p.institucion,
                p.institucion_direccion,
                p.institucion_cuit,
                p.institucion_posicion_fiscal,
                p.institucion_telefono,
                p.institucion_posicion_iibb,
                p.institucion_inicio_actividad,
                p.comprobante_tipo_factura,
                p.comprobante_tipo_nota_credito,
                p.comprobante_tipo_letra,
                p.comprobante_leyenda_pie

            FROM
                comprobantes_templates_parametrizaciones p
            LIMIT 1
	    ";

        return $this->entorno()->db()->consultar_fila($sql);
    }

    public function get_numeracion_faltante_afip($filtro = [])
    {
        $where = "";
        if (isset($filtro['id_punto_cobranza'])) {
            $where .= "AND sd.valor = " . $this->quote($filtro['id_punto_cobranza']);
        }

        $faltantes = [];

        $sql = " 
            SELECT
                s.id,
                s.codigo AS punto_cobranza_codigo,
                sd.valor  AS id_punto_cobranza,
	            pc.descripcion AS punto_cobranza_descripcion,
                uv.id as unidad_venta_id,
	            uv.descripcion as unidad_venta_descripcion
            FROM suscripciones_detalle sd
            JOIN suscripciones s ON sd.id_suscripcion = s.id
            JOIN unidades_ventas uv ON uv.id = s.id_unidad_venta
            JOIN suscripciones_parametros sp ON sp.id = sd.id_suscripcion_parametro
            JOIN puntos_cobranzas pc ON CAST ( pc.id AS TEXT  )  =  sd.valor 
            WHERE
                sp.codigo = '".constantes::SUSCRIPCION_PARAMETRO_CODIGO_PUNTO_COBRANZA."'
                AND s.finalizada_en IS NULL
                AND EXISTS (
                SELECT
                    s2.id
                FROM
                    suscripciones_detalle sd
                JOIN suscripciones s2 ON
                    sd.id_suscripcion = s2.id
                JOIN unidades_ventas uv ON
                    uv.id = s2.id_unidad_venta
                JOIN suscripciones_parametros sp ON
                    sp.id = sd.id_suscripcion_parametro
                WHERE
                    sp.codigo = '".constantes::SUSCRIPCION_PARAMETRO_CODIGO_ACEPTAAFIP."'
                    AND sd.valor = '".constantes::SUSCRIPCION_PARAMETRO_VOCABULARIO_TERMINO_SI."'
                    AND s2.finalizada_en IS NULL
                    AND s.id = s2.id)
                $where;
        ";

        foreach ($this->entorno()->db()->consultar($sql) as $r) {
            $obtenidos = $this->get_numeracion_faltante($r);

            if ($obtenidos) {
                $faltantes = array_merge($faltantes, $obtenidos);
            }
        }

        return $faltantes;
    }


    public function get_numeracion_faltante($filtro = [])
    {
        $faltantes = [];

        // Para cada conjunto de comprobantes de determinado template y punto de cobranza
        // busco si existe alguna numeración intermedia faltante entre el primero y el último
        $sql = "
                SELECT 
                    numero_faltante 
                FROM 
                    generate_series(1, ( 
                        SELECT MAX(numero_comprobante::INT) FROM comprobantes co WHERE co.id_comprobante_template = ".constantes::COMPROBANTE_TEMPLATE_COBRO." AND co.id_punto_cobranza = {$filtro['id_punto_cobranza']} ) 
                    ) AS serie_completa(numero_faltante)
                
                    WHERE serie_completa.numero_faltante NOT IN ( 
                        SELECT 
                            numero_comprobante::INT
                        FROM 
                            comprobantes  
                        WHERE 
                            id_comprobante_template = ".constantes::COMPROBANTE_TEMPLATE_COBRO." AND
                            id_punto_cobranza = {$filtro['id_punto_cobranza']} AND
                            numero_comprobante IS NOT NULL
                        ORDER BY 
                            numero_comprobante::INT
                    );
        ";

        foreach ($this->entorno()->db()->consultar($sql) as $r) {
            $faltante = [];
            $faltante['id_punto_cobranza'] = $filtro['id_punto_cobranza'];
            $faltante['punto_cobranza_descripcion'] = $filtro['punto_cobranza_descripcion'];
            $faltante['numero_faltante'] = $r['numero_faltante'];
            $faltante['punto_cobranza_codigo'] = $filtro['punto_cobranza_codigo'];
            $faltante['unidad_venta_id'] = $filtro['unidad_venta_id'];
            $faltante['unidad_venta_descripcion'] = $filtro['unidad_venta_descripcion'];
            $faltantes[] = $faltante;
        }

        return $faltantes;
    }
}
