<?php

namespace SIU\SQ\Lib\daos;

use SIU\SQ\Lib\daos\dao;
use SIU\SQ\Lib\errores\error_negocio;
use SIU\SQ\Uv\constantes;
use SIU\SQ\Lib\constantes as constantes_lib;
use SIU\SQ\Lib\lib;
use SIU\SQ\Uv\sq_uv;

class dao_procesos extends dao
{
    public function get($id)
    {
        $filtro['id'] = $id;
        $rs = $this->get_lista($filtro);
        if (count($rs) > 1) {
            throw new error_negocio("dao_procesos: "
                . "La consulta devolvio mas de un registro.");
        } else {
            return (isset($rs[0])) ? $rs[0] : null;
        }
    }

    public function get_lista($filtro = array(), $limit = "", $order_by = "")
    {
        $where = "";
        if (isset($filtro['id'])) {
            $where .= " AND p.id = " . $this->quote($filtro['id']);
        }
        if (isset($filtro['codigo'])) {
            $where .= " AND p.codigo = " . $this->quote($filtro['codigo']);
        }
        if (isset($filtro['modalidad'])) {
            $where .= " AND p.modalidad = " . $this->quote($filtro['modalidad']);
        }

        $sql = "SELECT
					p.id,
					p.codigo,
					p.descripcion,
					p.id_tipo_proceso,
					p.hora,
					p.prioridad,
					p.modalidad,
					p.estado,
					p.estado_actualizado_en
				FROM
					procesos p
				WHERE 1=1
					$where $order_by $limit;";
        return $this->entorno()->db()->consultar($sql);
    }

    public function get_procesos_ejecuciones($filtro = array(), $order_by = "", $limit = "", $con_detalle_estado = "", $con_detalle_proceso = "")
    {
        $select = "";
        $join = "";
        $where = "";
        if (isset($filtro['id'])) {
            $where .= " AND pe.id = " . $this->quote($filtro['id']);
        }
        if (isset($filtro['id_proceso'])) {
            $where .= " AND pe.id_proceso = " . $this->quote($filtro['id_proceso']);
        }
        if (isset($filtro['id_proceso_ejecucion_estado'])) {
            $where .= " AND pe.id_proceso_ejecucion_estado = " . $this->quote($filtro['id_proceso_ejecucion_estado']);
        }
        if (isset($filtro['fecha_desde'])) {
            $where .= " AND pe.iniciado_en::timestamp::date >= " . $this->quote($filtro['fecha_desde']);
        }
        if (isset($filtro['fecha_hasta'])) {
            $where .= " AND pe.iniciado_en::timestamp::date <= " . $this->quote($filtro['fecha_hasta']);
        }
        if (isset($filtro['total_a_procesar'])) {
            $where .= " AND pe.total_a_procesar = " . $this->quote($filtro['total_a_procesar']);
        }
        if (isset($filtro['procesados_ok'])) {
            $where .= " AND pe.procesados_ok = " . $this->quote($filtro['procesados_ok']);
        }
        if (isset($filtro['procesados_error'])) {
            $where .= " AND pe.procesados_error = " . $this->quote($filtro['procesados_error']);
        }
        if (isset($filtro['finalizado_en__not_null'])) {
            $where .= " AND pe.finalizado_en IS NOT NULL";
        }
        if (isset($filtro['id_venta'])) {
            $join .= " JOIN rep_ventas_nucleo rvn ON rvn.id_proceso_ejecucion = pe.id";
            $where .= " AND rvn.id = " . $this->quote($filtro['id_venta']);
        }
        if ($con_detalle_estado == 'true') {
            $select .= ", json_build_object(
									'id',pee.id,
									'codigo',pee.codigo,
									'descripcion',pee.descripcion
								) estado";
            $join .= " JOIN procesos_ejecuciones_estados pee ON pee.id = pe.id_proceso_ejecucion_estado";
        }
        if ($con_detalle_proceso == 'true') {
            $select .= ", json_build_object(
									'id',p.id,
									'codigo',p.codigo,
									'descripcion',p.descripcion
								) proceso";
            $join .= " JOIN procesos p ON p.id = pe.id_proceso";
        }
        $sql = "SELECT
					pe.id,
					pe.id_proceso,
					pe.iniciado_en,
					pe.finalizado_en,
					pe.id_proceso_ejecucion_estado,
					pe.total_a_procesar,
					pe.procesados_ok,
					pe.procesados_error
					$select
				FROM
					procesos_ejecuciones pe
				$join
				WHERE 1=1
					$where $order_by $limit";
        $ejecuciones = $this->entorno()->db()->consultar($sql);

        foreach ($ejecuciones as &$ejecucion) {
            if (isset($ejecucion['estado'])) {
                $ejecucion['estado'] = $this->json_decode_caracteres_especiales($ejecucion['estado']);
            }
            if (isset($ejecucion['proceso'])) {
                $ejecucion['proceso'] = $this->json_decode_caracteres_especiales($ejecucion['proceso']);
            }
        }

        return $ejecuciones;
    }

    public function get_procesos_ejecuciones_log($id_proceso, $filtro = array(), $order_by = "ORDER BY id", $limit = "")
    {
        $where = "";
        if (isset($filtro['ocurrido_en'])) {
            $where .= " AND ocurrido_en = " . $this->quote($filtro['ocurrido_en']);
        }
        if (isset($filtro['severidad'])) {
            $where .= " AND severidad = " . $this->quote($filtro['severidad']);
        }
        $sql = "SELECT
					id,
					id_proceso_ejecucion,
					ocurrido_en,
					severidad,
					componente,
					texto
				FROM
					procesos_ejecuciones_log
				WHERE
					id_proceso_ejecucion = " . $this->quote($id_proceso) . "
					$where $order_by $limit;";
        return $this->entorno()->db()->consultar($sql);
    }

    public function get_procesos_ejecuciones_estados($filtro = array())
    {
        $where = "";
        if (isset($filtro['id'])) {
            $where .= " AND pee.id = " . $this->quote($filtro['id']);
        }
        if (isset($filtro['codigo'])) {
            $where .= " AND pee.codigo = " . $this->quote($filtro['codigo']);
        }
        if (isset($filtro['descripcion'])) {
            $where .= " AND pee.descripcion = " . $this->quote($filtro['descripcion']);
        }

        $sql = "SELECT
					pee.id,
					pee.codigo,
					pee.descripcion
				FROM
					procesos_ejecuciones_estados pee
				WHERE 1=1
					$where";
        return $this->entorno()->db()->consultar($sql);
    }

    public function get_ultima_ejecucion($filtro = array())
    {
        $where = "";
        if (isset($filtro['id_proceso'])) {
            $where .= " AND id_proceso = " . $this->quote($filtro['id_proceso']);
        }

        $sql = "SELECT id_proceso, 
						iniciado_en, 
						finalizado_en, 
						id_proceso_ejecucion_estado 
				FROM procesos_ejecuciones 
				WHERE 1=1 
					$where 
					AND finalizado_en IS NOT NULL 
					AND id_proceso_ejecucion_estado = " . constantes::PROCESO_EJECUCION_ESTADO_OK . "
				ORDER BY iniciado_en 
				DESC LIMIT 1;";

        return $this->entorno()->db()->consultar_fila($sql);
    }

    public function get_procesos_parametros($filtro = array())
    {
        $where = "";
        if (isset($filtro['id'])) {
            $where .= " AND p.id = " . $this->quote($filtro['id']);
        }
        if (isset($filtro['codigo'])) {
            $where .= " AND p.codigo = " . $this->quote($filtro['codigo']);
        }

        $sql = "SELECT  p.id, 
						p.codigo, 
						p.descripcion, 
						p.tipo_dato,
                        p.tipo_dominio,
                        p.es_multivaluado,
                        p.cantidad_valores,
                        p.valor_default,
                        p.version_publicacion,
                        pd.id_proceso_parametro_vocabulario,
                        pd.proyector,
                        pd.query
				FROM procesos_parametros p
				     JOIN procesos_parametros_dominios pd ON pd.id_proceso_parametro = p.id 
				WHERE 1=1 
					$where 
				ORDER BY id DESC;";

        return $this->entorno()->db()->consultar($sql);
    }

    public function get_procesos_parametros_vocabularios($filtro = array())
    {
        $where = "";
        if (isset($filtro['id'])) {
            $where .= " AND v.id = " . $this->quote($filtro['id']);
        }
        if (isset($filtro['codigo'])) {
            $where .= " AND v.codigo = " . $this->quote($filtro['codigo']);
        }

        $sql = "SELECT  --v.id, 
						--v.codigo, 
						--v.descripcion,
						--vt.id as id_termino, 
                        --vt.codigo as codigo_termino,
                        --vt.descripcion as descripcion_termino
                        vt.id as id, 
                        vt.descripcion as valor
				FROM procesos_parametros_vocabularios v
				     JOIN procesos_parametros_vocabularios_terminos vt ON vt.id_proceso_parametro_vocabulario = v.id 
				WHERE 1=1 
					$where 
				ORDER BY id DESC;";

        return $this->entorno()->db()->consultar($sql);
    }

    public function get_parametro($id)
    {
        $filtro['id'] = $id;
        $rs = $this->get_procesos_parametros($filtro);
        if (count($rs) > 1) {
            throw new error_negocio(__METHOD__ . ": "
                . "La consulta devolvio mas de un registro.");
        } else {
            return (isset($rs[0])) ? $rs[0] : null;
        }
    }

    public function get_valores_por_parametro($id)
    {
        $parametro = $this->get_parametro($id);

        // Según el tipo de dominio realizar la acción
        if (!is_null($parametro['query']) && !is_null($parametro['proyector'])) {
            $query = str_replace("%proyector%", "", $parametro['query']);
            return $this->entorno()->db()->consultar($query);
        } elseif ($parametro['id_proceso_parametro_vocabulario']) {
            return $this->get_procesos_parametros_vocabularios(['id' => $parametro['id_proceso_parametro_vocabulario'] ]);
        } else {
            $msg = "El parametro con el id: '" . $id . "' no tiene definido un dominio.";
            lib::entorno()->log()->error(__METHOD__ . ": " . $msg);
            throw new error_negocio($msg);
        }
    }

    public function get_procesos_parametrizaciones($filtro)
    {
        $where = "";
        if (isset($filtro['id'])) {
            $where .= " AND ppz.id = " . $this->quote($filtro['id']);
        }
        if (isset($filtro['id_proceso'])) {
            $where .= " AND id_proceso = " . $this->quote($filtro['id_proceso']);
        }
        if (isset($filtro['id_proceso_parametro'])) {
            $where .= " AND id_proceso_parametro = " . $this->quote($filtro['id_proceso_parametro']);
        }

        $sql = "SELECT  ppz.id, 
						ppz.id_proceso, 
						ppz.id_proceso_parametro,
						pps.descripcion
				FROM procesos_parametrizaciones ppz
				LEFT JOIN  procesos_parametros pps on ppz.id_proceso_parametro = pps.id
				WHERE 1=1 
					$where 
				ORDER BY id DESC;";

        return $this->entorno()->db()->consultar($sql);
    }

    public function get_valores_procesos_parametrizaciones($id_proceso_parametrizacion)
    {
        $parametros = $this->get_procesos_parametrizaciones(['id' => $id_proceso_parametrizacion]);
        $salida = [];
        foreach ($parametros as $parametro) {
            $valores = $this->get_valores_por_parametro($parametro['id_proceso_parametro']);
            foreach ($valores as $valor) {
                $salida[] = ['id' => $valor['id'], 'valor' => $valor['valor']];
            }
        }
        return $salida;
    }

    public function get_tareas_programadas()
    {
        $sql = "SELECT
                    ptp.id,
                    ptp.id_proceso,
                    ptp.tipo_frecuencia,
                    ptp.dia,
                    ptp.hora,
                    ptp.intervalo,
                    proc.descripcion as proceso_descripcion
                    ,(SELECT descripcion FROM procesos_parametros_vocabularios_terminos ppvt WHERE ppvt.id = cast(ptpv.valor as integer)) as parametro
				FROM
				    procesos_tareas_programadas ptp
				    JOIN procesos proc ON proc.id = ptp.id_proceso
				    LEFT OUTER JOIN procesos_tareas_programadas_valores ptpv ON ptpv.id_proceso_tarea_programada = ptp.id       
				        
				ORDER BY
				    ptp.id_proceso;";

        return $this->entorno()->db()->consultar($sql);
    }

    public function get_tareas_programadas_valores($filtro)
    {
        $where = "";

        if (isset($filtro['id'])) {
            $where .= " AND ptpv.id = " . $this->quote($filtro['id']);
        }
        if (isset($filtro['id_proceso_tarea_programada'])) {
            $where .= " AND ptpv.id_proceso_tarea_programada = " . $this->quote($filtro['id_proceso_tarea_programada']);
        }
        if (isset($filtro['id_proceso'])) {
            $where .= " AND pp.id_proceso = " . $this->quote($filtro['id_proceso']);
        }

        $sql = "SELECT
                    pp.id as id_parametrizacion,
                    pp.id_proceso as id_proceso,
                    ptpv.id,
                    ptpv.id_proceso_tarea_programada,
                    ptpv.id_proceso_parametrizacion,
                    ptpv.valor,

                    param.id as id_proceso_parametro,
                    param.descripcion as parametro_descripcion
                    
				FROM
				    procesos_tareas_programadas_valores ptpv
				    JOIN procesos_parametrizaciones pp ON pp.id = ptpv.id_proceso_parametrizacion
				    JOIN procesos_parametros param ON param.id = pp.id_proceso_parametro
				WHERE 1=1
				    $where
				ORDER BY
				    ptpv.id;";

        return $this->entorno()->db()->consultar($sql);
    }

    public function get_descripcion_parametro($filtro)
    {
        $where = $from = "";
        if (isset($filtro['id'])) {
            $where .= " AND p.id = " . $this->quote($filtro['id']);
        }
        if (isset($filtro['codigo'])) {
            $where .= " AND p.codigo = " . $this->quote($filtro['codigo']);
        }
        if (isset($filtro['id_proceso_parametrizacion'])) {
            $where .= " AND pp.id = " . $this->quote($filtro['id_proceso_parametrizacion']);
            $from .= " JOIN procesos_parametrizaciones pp ON pp.id_proceso_parametro = p.id ";
        }

        $sql = "SELECT
                    p.id, 
					p.codigo, 
					p.descripcion, 
					p.tipo_dato,
                    p.tipo_dominio,
                    p.es_multivaluado,
                    p.cantidad_valores,
                    p.valor_default,
                    p.version_publicacion
				FROM
				    procesos_parametros p
    				$from
				WHERE 1=1 
					$where 
				ORDER BY id DESC;";

        return $this->entorno()->db()->consultar($sql);
    }

    public function get_listado($origen)
    {
        $from = $select = '';
        if (isset($origen)) {
            if($origen == constantes_lib::ORIGEN_NUCLEO) {
                $select = ",(CASE WHEN t_ptpv_uv.id is not null AND t_ptpv_informe.id is not null THEN
                                        concat(t_p.descripcion,' || ', vt_informe.descripcion, ' || ', t_uv.descripcion)
                                  WHEN t_ptpv_uv.id is not null AND t_ptpv_informe.id is null THEN
                                        concat(t_p.descripcion,' || ', t_uv.descripcion)
                                  WHEN t_ptpv_uv.id is null AND t_ptpv_informe.id is not null THEN
                                        concat(t_p.descripcion,' || ', vt_informe.descripcion)                
                                  ELSE t_p.descripcion
                   END) as id_proceso_nombre";

                $from = " LEFT JOIN procesos_tareas_programadas_valores as t_ptpv_informe
                            JOIN procesos_parametrizaciones as t_pps_informe on t_pps_informe.id = t_ptpv_informe.id_proceso_parametrizacion
                            JOIN procesos_parametros as t_ppa_informe on t_ppa_informe.id = t_pps_informe.id_proceso_parametro AND t_ppa_informe.codigo ='INFORME'
                            JOIN procesos_parametros_vocabularios_terminos vt_informe ON vt_informe.id = cast(t_ptpv_informe.valor as int)
                          ON t_ptpv_informe.id_proceso_tarea_programada =  t_ptp.id

                          LEFT JOIN procesos_tareas_programadas_valores as t_ptpv_uv
                            JOIN procesos_parametrizaciones as t_pps_uv on t_pps_uv.id = t_ptpv_uv.id_proceso_parametrizacion
                            JOIN procesos_parametros as t_ppa_uv on t_ppa_uv.id = t_pps_uv.id_proceso_parametro AND t_ppa_uv.codigo ='UNIDVENTA'
                            JOIN unidades_ventas t_uv ON t_uv.id = cast(t_ptpv_uv.valor as int)
                          ON t_ptpv_uv.id_proceso_tarea_programada =  t_ptp.id";

            } else {
                $select = " ,(CASE WHEN t_ptpv_ts.id is not null THEN
            	                concat(t_p.descripcion,' || ', t_ts.descripcion)
            	            ELSE t_p.descripcion
                            END) as id_proceso_nombre ";
                $from = " LEFT JOIN procesos_tareas_programadas_valores as t_ptpv_ts
                            JOIN procesos_parametrizaciones as t_pps_ts on t_pps_ts.id = t_ptpv_ts.id_proceso_parametrizacion
                            JOIN procesos_parametros as t_ppa_ts on t_ppa_ts.id = t_pps_ts.id_proceso_parametro
                            JOIN  tipos_servicios as t_ts ON t_ts.id = cast(t_ptpv_ts.valor as int)
                        ON t_ptpv_ts.id_proceso_tarea_programada =  t_ptp.id AND t_ppa_ts.codigo ='TIPOSERVIC'";
            }
        }

        $sql = "
        SELECT
			t_ptp.id,
			t_p.id as id_proceso,
			t_ptp.tipo_frecuencia,
			t_ptp.dia,
			t_ptp.hora,
			t_ptp.intervalo,
            (SELECT COUNT(*) FROM procesos_tareas_programadas_valores ptpv WHERE ptpv.id_proceso_tarea_programada = t_ptp.id) AS cantidad_parametrizaciones_seteadas,
            (SELECT COUNT(*) FROM procesos_parametrizaciones pp WHERE pp.id_proceso = t_p.id) AS cantidad_parametrizaciones
            $select
		FROM
            procesos_tareas_programadas as t_ptp
        LEFT JOIN procesos as t_p ON t_ptp.id_proceso = t_p.id
        $from
		ORDER BY t_ptp.tipo_frecuencia";
        lib::entorno()->log()->debug(__METHOD__ . ": " . $sql);
        return $this->entorno()->db()->consultar($sql);
    }
}
