<?php

namespace SIU\SQ\Lib\modelo;

use SIU\SQ\Lib\lib;
use SIU\SQ\Uv\constantes;
use SIU\SQ\Lib\errores\error_negocio;

class catalogo extends \SIU\SQ\Lib\entorno\catalogo
{
    public function version()
    {
        return new version($this->entorno);
    }

    public function notificacion($id = null, $dao_notificaciones = null)
    {
        return new notificacion($this->entorno, $id, $dao_notificaciones);
    }

    public function parametro_sistema($id = null)
    {
        return new parametro_sistema($this->entorno, $id);
    }

    public function facturador($id_tipo_servicio = null)
    {
        // Según si el tipo de servicio devenga o no definir que facturador tomar
        $datos_tipo_servicio = lib::catalogo_daos()->facturador()->get_tipos_servicios(['id' => $id_tipo_servicio]);
        if (empty($datos_tipo_servicio)) {
            $msg = "Error al facturar. El tipo de servicio con el id: '" . $id_tipo_servicio . "' no es válido.";
            lib::entorno()->log()->error(__METHOD__ . ": " . $msg);
            throw new error_negocio($msg);
        }

        switch ($id_tipo_servicio) {
            case constantes::ID_TIPO_SERVICIO_EXTENSION:
                lib::entorno()->log()->info("Facturador TS Extension");
                return new facturador_cursos_extension($this->entorno);
                break;

            case constantes::ID_TIPO_SERVICIO_POSGRADO:
                lib::entorno()->log()->info("Facturador TS Posgrado");
                return new facturador_posgrados($this->entorno);
                break;

            case constantes::ID_TIPO_SERVICIO_GRADO:
                lib::entorno()->log()->info("Facturador TS Grado Virtual");
                return new facturador_grado_virtual($this->entorno);
                break;

            default:
                throw new error_negocio("Facturador: No se pudo determinar el tipo de servicio para el facturador.");

        }
    }


    public function facturador_alarma($id = null)
    {
        return new facturador_alarma($this->entorno, $id);
    }
    
    public function reloj($id_reloj = null, $id_periodo_fact_excepciones = null)
    {
        return new reloj($this->entorno, $id_reloj, $id_periodo_fact_excepciones);
    }
    
    public function reloj_instancia($id_servicio_instancia)
    {
        return new reloj_instancia($this->entorno, $id_servicio_instancia);
    }
    
    public function comprobante($id = null)
    {
        return new comprobante($this->entorno, $id);
    }

    public function comprobante_template($id = null)
    {
        return new comprobante_template($this->entorno, $id);
    }

    public function procesos_ejecuciones($id = null)
    {
        return new procesos_ejecuciones($this->entorno, $id);
    }

    public function venta_info_personalizada($id_venta = null)
    {
        return new venta_info_personalizada($this->entorno, $id_venta);
    }

    public function log_interaccion_externa($id = null)
    {
        return new log_interaccion_externa($this->entorno, $id);
    }
}
