<?php

namespace SIU\SQ\Lib\modelo;

use SIU\SQ\Lib\constantes;
use SIU\SQ\Lib\errores\error_negocio;
use SIU\SQ\lib\lib;
use SIU\SQ\Lib\db\sql;
use SIU\SQ\Lib\errores\error_not_found;

class comprobante_template extends modelo
{
    protected function inicializar()
    {
        $this->datos['id'] = null;
        $this->datos['codigo'] = null;
        $this->datos['descripcion'] = null;
        $this->datos['tipo_comprobante'] = null;
        $this->datos['modelo'] = null;
        $this->datos['numeracion'] = null;
        $this->datos['estado'] = null;
        $this->datos['estado_actualizado_en'] = null;
    }
    
    protected function cargar($id)
    {
        $this->datos = lib::catalogo_daos()->comprobantes()->get_template($id);
        if (empty($this->datos)) {
            throw new error_not_found(__METHOD__ . " No se pudo recuperar el comprobante_template ID $id");
        }
    }

    protected function validar()
    {
        $campos_obligatorios = ['codigo','descripcion','tipo_comprobante','modelo'];
        $this->validar_campos_obligatorios($this->datos, $campos_obligatorios, __METHOD__);
    }

    public function guardar()
    {
        $this->validar();
        if (empty(lib::catalogo_daos()->comprobantes()->get_template($this->get_id()))) {
            $this->grabar();
        } else {
            $this->actualizar();
        }
        return $this->get_id();
    }
    
    private function grabar()
    {
        $datos = [
            'id'					=> $this->get_id(),
            'codigo'				=> $this->datos['codigo'],
            'descripcion'			=> $this->datos['descripcion'],
            'tipo_comprobante'		=> $this->datos['tipo_comprobante'],
            'modelo'				=> $this->datos['modelo'],
            'numeracion'			=> $this->datos['numeracion'],
            'estado'				=> $this->datos['estado'],
            'estado_actualizado_en'	=> $this->datos['estado_actualizado_en']
        ];
        $sql = sql::sql_array_a_insert('comprobantes_templates', $datos, $this->entorno()->db());
        $this->entorno()->db()->ejecutar($sql);
    }
    
    private function actualizar()
    {
        $datos = [
            'codigo'				=> $this->datos['codigo'],
            'descripcion'			=> $this->datos['descripcion'],
            'tipo_comprobante'		=> $this->datos['tipo_comprobante'],
            'modelo'				=> $this->datos['modelo'],
            'numeracion'			=> $this->datos['numeracion'],
            'estado'				=> $this->datos['estado'],
            'estado_actualizado_en'	=> $this->datos['estado_actualizado_en']
        ];
        $where = ['id' => $this->get_id()];
        $sql = sql::sql_array_a_update('comprobantes_templates', $datos, $where, $this->entorno()->db());
        $this->entorno()->db()->ejecutar($sql);
    }
    
    public function avanzar_numeracion()
    {
        if (empty($this->datos['numeracion'])) {
            $this->datos['numeracion'] = 1;
        } else {
            $this->datos['numeracion'] = $this->datos['numeracion'] + 1;
        }
    }
    
    //--SETTERS
    
    public function set_id($id)
    {
        $this->datos['id'] = $id;
    }
    
    public function set_codigo($codigo)
    {
        $this->datos['codigo'] = $codigo;
    }
    
    public function set_descripcion($descripcion)
    {
        $this->datos['descripcion'] = $descripcion;
    }
    public function set_tipo_comprobante($tipo_comprobante)
    {
        $this->datos['tipo_comprobante'] = $tipo_comprobante;
    }
    
    public function set_modelo($modelo)
    {
        $this->datos['modelo'] = $modelo;
    }

    public function set_numeracion($numeracion)
    {
        $this->datos['numeracion'] = $numeracion;
    }
    
    public function set_estado($estado)
    {
        if (is_null($this->datos['estado']) || $this->datos['estado'] != $estado) {
            $this->datos['estado_actualizado_en'] = 'now()';
        }
        $this->datos['estado'] = $estado;
    }
    
    //--GETTERS
    
    public function get_numeracion()
    {
        return $this->datos['numeracion'];
    }
    
}
