<?php
namespace SIU\SQ\Lib\modelo;

use SIU\SQ\Lib\db\sql;
use SIU\SQ\Lib\lib;
use SIU\SQ\Lib\modelo\modelo;
use SIU\SQ\Lib\errores\error_negocio;
use SIU\SQ\Lib\constantes;

/**
 * Description of log_interaccion_externa
 */
class log_interaccion_externa extends modelo
{
    protected function inicializar()
    {
        $this->datos['id'] = null;
        $this->datos['tipo'] = null;
        $this->datos['sistema_externo'] = null;
        $this->datos['ocurrido_en'] = 'now()';
        $this->datos['mensaje'] = null;
        $this->datos['recurso'] = null;
        $this->datos['verbo'] = null;
        $this->datos['codigo_procesamiento'] = constantes::LOG_INTERACCION_EXTERNA_CODIGO_PROCESAMIENTO_NUEVO;
        $this->datos['mensaje_procesamiento'] = '';
    }

    protected function cargar($id)
    {
        $this->datos = lib::catalogo_daos()->log_interacciones_externas()->get($id);
        if ($this->datos == null) {
            throw new error_negocio("El log de interacción externa que intenta procesar no es válido");
        }
    }

    protected function validar()
    {
        // Controlar que esten cargados los datos basicos
        $campos_obligatorios = array(
            'tipo',
            'sistema_externo',
            'ocurrido_en',
            'mensaje'
        );
        $this->validar_campos_obligatorios($this->datos, $campos_obligatorios, __CLASS__);
    }

    public function guardar()
    {
        $this->validar();

        if ($this->get_id() != null) {
            $this->actualizar_log();
        } else {
            $this->grabar_log();
        }
    }

    protected function grabar_log()
    {
        $datos = array(
            'tipo' => $this->get_tipo(),
            'sistema_externo' => $this->get_sistema_externo(),
            'ocurrido_en' => $this->get_ocurrido_en(),
            'mensaje' => $this->get_mensaje(),
            'recurso' => $this->get_recurso(),
            'verbo' => $this->get_verbo(),
            'codigo_procesamiento' => $this->get_codigo_procesamiento(),
            'mensaje_procesamiento' => $this->get_mensaje_procesamiento(),
        );
        $sql = sql::sql_array_a_insert('log_interacciones_externas', $datos, $this->entorno()->db());
        $this->entorno()->db()->ejecutar($sql);
        // Obtener el id generado
        $sql_consumo_generado = "SELECT id FROM log_interacciones_externas
									WHERE id = (SELECT currval('log_interacciones_externas_id_seq'))";
        $consumo_generado = $this->entorno()->db()->consultar_fila($sql_consumo_generado);

        $this->datos['id']		= $consumo_generado['id'];
    }

    public function actualizar_log()
    {
        $datos = array(
            'ocurrido_en' => 'now()',
            'mensaje' => $this->get_mensaje(),
            'codigo_procesamiento' => $this->get_codigo_procesamiento(),
            'mensaje_procesamiento' => $this->get_mensaje_procesamiento()
        );
        $where = array('id' => $this->get_id());
        $sql = sql::sql_array_a_update('log_interacciones_externas', $datos, $where, $this->entorno()->db());
        $this->entorno()->db()->ejecutar($sql);
    }

    // ----- GETTERS ---- //

    public function get_tipo()
    {
        return $this->datos['tipo'];
    }

    public function get_sistema_externo()
    {
        return $this->datos['sistema_externo'];
    }

    public function get_ocurrido_en()
    {
        return $this->datos['ocurrido_en'];
    }

    public function get_mensaje()
    {
        return $this->datos['mensaje'];
    }

    public function get_recurso()
    {
        return $this->datos['recurso'];
    }

    public function get_verbo()
    {
        return $this->datos['verbo'];
    }

    public function get_codigo_procesamiento()
    {
        return $this->datos['codigo_procesamiento'];
    }

    public function get_mensaje_procesamiento()
    {
        return $this->datos['mensaje_procesamiento'];
    }

    // ----- SETTERS ---- //

    public function set_tipo($tipo)
    {
        $this->datos['tipo'] = $tipo;
    }

    public function set_sistema_externo($sistema_externo)
    {
        $this->datos['sistema_externo'] = $sistema_externo;
    }

    public function set_ocurrido_en($fecha)
    {
        $this->datos['ocurrido_en'] = $fecha;
    }

    public function set_mensaje($mensaje)
    {
        $this->datos['mensaje'] = $mensaje;
    }

    public function set_recurso($recurso)
    {
        $this->datos['recurso'] = $recurso;
    }

    public function set_verbo($verbo)
    {
        $this->datos['verbo'] = $verbo;
    }

    private function set_codigo_procesamiento($codigo)
    {
        $this->datos['codigo_procesamiento'] = $codigo;
    }

    private function set_mensaje_procesamiento($mensaje)
    {
        $this->datos['mensaje_procesamiento'] = $mensaje;
    }
}
