<?php

namespace SIU\SQ\Lib\modelo;

use SIU\SQ\Lib\constantes;
use SIU\SQ\Lib\db\sql;
use SIU\SQ\Lib\entorno\configurador;
use SIU\SQ\Lib\errores\error;
use SIU\SQ\Lib\errores\error_negocio;
use SIU\SQ\Lib\errores\error_not_found;
use SIU\SQ\Lib\lib;

class procesos_ejecuciones extends modelo
{
    protected function inicializar()
    {
        $this->datos['id']				    = null;
        $this->datos['id_proceso']		    = null;
        $this->datos['total_a_procesar']    = 0;
        $this->datos['procesados_ok']		= 0;
        $this->datos['procesados_error']	= 0;
        $this->datos['iniciado_en']         = "now()";
        $this->datos['finalizado_en']		= null;
        $this->datos['id_proceso_ejecucion_estado']	= null;
        $this->datos['descripcion']	= null;
    }

    protected function cargar($id_ejecucion)
    {
        $proceso_ejecucion = lib::catalogo_daos()->procesos()->get_procesos_ejecuciones(['id' => $id_ejecucion]);
        if (!empty($proceso_ejecucion)) {
            $this->datos = $proceso_ejecucion[0];
        } else {
            $msg = "La ejecución del proceso '" . $id_ejecucion . "'' que intenta cargar no existe.";
            lib::entorno()->log()->error(__METHOD__ . ": " . $msg);
            throw new error_not_found($msg);
        }
    }

    protected function validar()
    {
        $campos_obligatorios = array(
            'id_proceso',
            'id_proceso_ejecucion_estado'
        );

        $this->validar_campos_obligatorios($this->datos, $campos_obligatorios, __CLASS__);
    }

    public function guardar()
    {
        $this->validar();

        if ($this->datos['id'] !== null) {
            $this->actualizar();
        } else {
            $this->grabar();
        }
        return $this->datos['id'];
    }

    private function grabar()
    {
        $datos = [
            'id_proceso'                    => $this->datos['id_proceso'],
            'total_a_procesar'              => $this->datos['total_a_procesar'],
            'procesados_ok'                 => $this->datos['procesados_ok'],
            'procesados_error'              => $this->datos['procesados_error'],
            'iniciado_en'                   => $this->datos['iniciado_en'],
            'finalizado_en'                 => $this->datos['finalizado_en'],
            'id_proceso_ejecucion_estado'   => $this->datos['id_proceso_ejecucion_estado'],
            'descripcion'               => $this->datos['descripcion']
        ];

        $sql = sql::sql_array_a_insert('procesos_ejecuciones', $datos, $this->entorno()->db());
        $this->entorno()->db()->ejecutar($sql);

        $seq = $this->entorno()->db()->consultar_fila("SELECT currval('procesos_ejecuciones_id_seq') as seq;");
        $this->datos['id'] = $seq['seq'];
    }

    private function actualizar()
    {
        $datos = [
            'id_proceso'                    => $this->datos['id_proceso'],
            'total_a_procesar'              => $this->datos['total_a_procesar'],
            'procesados_ok'                 => $this->datos['procesados_ok'],
            'procesados_error'              => $this->datos['procesados_error'],
            'iniciado_en'                   => $this->datos['iniciado_en'],
            'finalizado_en'                 => $this->datos['finalizado_en'],
            'id_proceso_ejecucion_estado'   => $this->datos['id_proceso_ejecucion_estado'],
            'descripcion'               => $this->datos['descripcion']
        ];
        $where = ['id' => $this->get_id()];
        $sql = sql::sql_array_a_update('procesos_ejecuciones', $datos, $where, $this->entorno()->db());
        $this->entorno()->db()->ejecutar($sql);
    }

    public function set_id_proceso($id)
    {
        $this->datos['id_proceso'] = $id;
    }

    public function get_id_proceso()
    {
        return $this->datos['id_proceso'];
    }

    public function set_total_a_procesar($total_a_procesar)
    {
        $this->datos['total_a_procesar'] = $total_a_procesar;
    }

    public function get_total_a_procesar()
    {
        return $this->datos['total_a_procesar'];
    }

    public function set_procesados_ok($id)
    {
        $this->datos['procesados_ok'] = $id;
    }

    public function get_procesados_ok()
    {
        return $this->datos['procesados_ok'];
    }

    public function set_procesados_error($id)
    {
        $this->datos['procesados_error'] = $id;
    }

    public function get_procesados_error()
    {
        return $this->datos['procesados_error'];
    }

    public function set_finalizado_en($id)
    {
        $this->datos['finalizado_en'] = $id;
    }

    public function get_finalizado_en()
    {
        return $this->datos['finalizado_en'];
    }

    public function set_descripcion($descripcion)
    {
        $this->datos['descripcion'] = $descripcion;
    }

    public function get_descripcion()
    {
        return $this->datos['descripcion'];
    }


    public function set_id_proceso_ejecucion_estado($id)
    {
        $this->datos['id_proceso_ejecucion_estado'] = $id;
    }

    public function get_id_proceso_ejecucion_estado()
    {
        return $this->datos['id_proceso_ejecucion_estado'];
    }
}
