<?php
namespace SIU\SQ\Lib\procesos;

use SIU\SQ\Lib\constantes;
use SIU\SQ\Lib\lib;
use SIU\SQ\Lib\errores\error;
use SIU\SQ\Lib\errores\error_db;
use SIU\SQ\Lib\errores\error_negocio;

class proceso_confirmar_facturacion extends proceso
{
    protected $cantidad_ventas_generadas;
    protected $importe_ventas_generadas;
    protected $ventas_generadas;
    protected $id_tipo_servicio;
    protected $datos_vencimientos;

    public function ejecutar_especifico()
    {
        try {
            $facturador = lib::catalogo_modelo()->facturador($this->id_tipo_servicio);
            $facturador->set_proceso($this);

            $facturador->set_dia_primer_vencimiento($this->datos_vencimientos['id_reloj_1er_vencimiento'], $this->datos_vencimientos['dia_1er_vencimiento']);
            if (isset($this->datos_vencimientos['id_reloj_2do_vencimiento'])){
                $facturador->set_dia_segundo_vencimiento($this->datos_vencimientos['id_reloj_2do_vencimiento'], $this->datos_vencimientos['dia_2do_vencimiento']);
            }
            $facturador->confirmar();

            $this->cantidad_ventas_generadas	= $facturador->get_cantidad_ventas_generadas();
            $this->importe_ventas_generadas		= $facturador->get_importe_ventas_generadas();
            $this->ventas_generadas				= $facturador->get_ventas_generadas();
            
            $this->agregar_notificacion("Ha finalizado el proceso de confirmación de la facturación", 'info');
            $this->finalizar_registro_ejecucion(constantes::PROCESO_EJECUCION_ESTADO_OK);
        } catch (error_db $e) {
            $this->agregar_notificacion($e->getMessage(), 'error');
            $this->finalizar_registro_ejecucion(constantes::PROCESO_EJECUCION_ESTADO_ERROR);
            throw $e;
        } catch (error_negocio $e) {
            $this->agregar_notificacion($e->getMessage(), 'error');
            $this->finalizar_registro_ejecucion(constantes::PROCESO_EJECUCION_ESTADO_ERROR);
            throw $e;
        } catch (error $e) {
            $this->agregar_notificacion($e->getMessage(), 'error');
            $this->finalizar_registro_ejecucion(constantes::PROCESO_EJECUCION_ESTADO_ERROR);
            throw $e;
        } catch (\Exception $e) {
            $this->agregar_notificacion($e->getMessage(), 'error');
            $this->finalizar_registro_ejecucion(constantes::PROCESO_EJECUCION_ESTADO_ERROR);
            throw $e;
        }
    }

    public function set_id_tipo_servicio($id_tipo_servicio)
    {
        $this->id_tipo_servicio = $id_tipo_servicio;
    }

    public function get_cantidad_ventas_generadas()
    {
        return $this->cantidad_ventas_generadas;
    }

    public function get_importe_ventas_generadas()
    {
        return $this->importe_ventas_generadas;
    }

    public function get_ventas_generadas()
    {
        return $this->ventas_generadas;
    }

    public function set_datos_vencimiento($datos)
    {
        $this->datos_vencimientos = $datos;
    }
}
