<?php

namespace SIU\SQ\Lib\sistemas_externos;

use SIU\SQ\Lib\errores\error;
use SIU\SQ\Lib\lib;
use SIU\SQ\Lib\constantes;

class mercado_pago extends gestor_pago
{
    /**
     * @param $id_referencia (Id externo de la cobranza)
     * @throws error
     */
    public function anular_cupon($id_referencia)
    {
        try {
            $r = lib::entorno()->get_cliente('sq_pagos_backend')->put('mercado_pago/expirar_payment/' . $id_referencia);
        } catch (\GuzzleHttp\Exception\ClientException $ex) {
            $msg = $ex->getResponse()->getBody()->getContents() . ' :' . $id_referencia;
            lib::entorno()->log()->err(__METHOD__ . ": " . $msg);
            throw new error($msg);
        } catch (\Exception $ex) {
            $msg = $ex->getMessage() . ' :' . $id_referencia;
            lib::entorno()->log()->err(__METHOD__ . ": " . $msg);
            throw new error($msg);
        }
    }

    /**
     * @param $id_referencia (Id externo de la cobranza)
     * @throws error
     */
    public function devolver_pago($id_referencia)
    {
        try {
            lib::entorno()->get_cliente('sq_pagos_backend')->put('mercado_pago/devolver_payment/' . $id_referencia);
        } catch (\GuzzleHttp\Exception\ClientException $ex) {
            $msg = $ex->getResponse()->getBody()->getContents() . ' :' . $id_referencia;
            lib::entorno()->log()->err(__METHOD__ . ": " . $msg);
            throw new error($msg);
        } catch (\Exception $ex) {
            $msg = $ex->getMessage() . ' :' . $id_referencia;
            lib::entorno()->log()->err(__METHOD__ . ": " . $msg);
            throw new error($msg);
        }
    }

    /**
     * @param $id_referencia (Id de Merchant Order)
     * @throws error
     */
    public function expirar_url_de_pago($id_referencia)
    {
        try {
            $merchant_order = $this->obtener_merchant_order_mp($id_referencia);
            $id_preferencia = $merchant_order['preference_id'];
            lib::entorno()->get_cliente('sq_pagos_backend')->put('mercado_pago/expirar_preferencia/' . $id_preferencia);
        } catch (\GuzzleHttp\Exception\ClientException $ex) {
            $msg = $ex->getResponse()->getBody()->getContents() . ' :' . $id_preferencia;
            lib::entorno()->log()->err(__METHOD__ . ": " . $msg);
            throw new error($msg);
        } catch (\Exception $ex) {
            $msg = $ex->getMessage() . ' :' . $id_preferencia;
            lib::entorno()->log()->err(__METHOD__ . ": " . $msg);
            throw new error($msg);
        }
    }

    /**
     * @param $id_referencia (Id de Merchant Order)
     * @return mixed
     * @throws error
     */
    public function obtener_datos_del_pago($id_referencia)
    {
        try {
            $response = lib::entorno()->get_cliente('sq_pagos_backend')->get('mercado_pago/payment/' . $id_referencia);
            $datos = json_decode($response->getBody()->getContents(), true);
            return $datos;
        } catch (\GuzzleHttp\Exception\ClientException $ex) {
            $msg = $ex->getResponse()->getBody()->getContents() . ' :' . $id_referencia;
            lib::entorno()->log()->err(__METHOD__ . ": " . $msg);
            throw new error($msg);
        } catch (\Exception $ex) {
            $msg = $ex->getMessage() . ' :' . $id_referencia;
            lib::entorno()->log()->err(__METHOD__ . ": " . $msg);
            throw new error($msg);
        }
    }

    /**
     * @param $parametros
     * @return mixed
     * @throws error
     */
    public function obtener_datos_de_pagos($parametros)
    {
        try {
            $payments = [];

            // Consulta inicial
            $response = $this->entorno()->get_cliente('sq_pagos_backend')->get('mercado_pago/payment', ['query' => $parametros]);
            $datos_response = json_decode($response->getBody()->getContents(), true);

            // Guardo primera tanda de payments recibidos.
            $payments = $datos_response['results'];

            // Hay paginado?
            if (isset($datos_response['paging'])) {
                $paging = $datos_response['paging'];
            }

            // Vuelvo a realizar n consultas para traer todo.
            if (!empty($paging) and ($paging['limit'] < $paging['total'])) {
                $cant_paginas = ceil($paging['total'] / $paging['limit']);
                for ($pagina = 1; $pagina <= $cant_paginas; $pagina++) {
                    $parametros['offset'] = $pagina * $paging['limit'];

                    $response = $this->entorno()->get_cliente('sq_pagos_backend')->get('mercado_pago/payment', ['query' => $parametros]);
                    $datos_response = json_decode($response->getBody()->getContents(), true);

                    // Voy sumando payments de siguientes tandas
                    $payments = array_merge($payments, $datos_response['results']);
                }
            }

            // devuelvo todos los payments
            return $payments;
        } catch (\GuzzleHttp\Exception\ClientException $ex) {
            if ($ex->getCode() != 423) {
                $msg = $ex->getResponse()->getBody()->getContents();
                lib::entorno()->log()->err(__METHOD__ . ": " . $msg);
                throw new error($msg);
            }
        } catch (\Exception $ex) {
            $msg = $ex->getMessage();
            lib::entorno()->log()->err(__METHOD__ . ": " . $msg);
            throw new error($msg);
        }
    }

    //------------------------------------------------------------------------------------------------------------------

    public function obtener_merchant_order_mp($id_merchant_order)
    {
        try {
            $response = lib::entorno()->get_cliente('sq_pagos_backend')->get('mercado_pago/merchant_order/' . $id_merchant_order);
            $merchant_order = json_decode($response->getBody()->getContents(), true);
            return $merchant_order;
        } catch (\GuzzleHttp\Exception\ClientException $ex) {
            $msg = $ex->getResponse()->getBody()->getContents() . ' :' . $id_merchant_order;
            lib::entorno()->log()->err(__METHOD__ . ": " . $msg);
            throw new error($msg);
        } catch (\Exception $ex) {
            $msg = $ex->getMessage() . ' :' . $id_merchant_order;
            lib::entorno()->log()->err(__METHOD__ . ": " . $msg);
            throw new error($msg);
        }
    }

    /**
     * @throws error
     */
    public function get_metodos_pagos()
    {
        try {
            $response = lib::entorno()->get_cliente('sq_pagos_backend')->get('mercado_pago/metodos_pagos');
            $metodos_pagos = json_decode($response->getBody()->getContents(), true);
            return $metodos_pagos;
        } catch (\GuzzleHttp\Exception\ClientException $ex) {
            $msg = $ex->getResponse()->getBody()->getContents();
            lib::entorno()->log()->err(__METHOD__ . ": " . $msg);
            throw new error($msg);
        } catch (\Exception $ex) {
            $msg = $ex->getMessage();
            lib::entorno()->log()->err(__METHOD__ . ": " . $msg);
            throw new error($msg);
        }
    }
}
