<?php

namespace SIU\SQ\Lib\sistemas_externos;

use SIU\SQ\Lib\errores\error;
use SIU\SQ\Lib\lib;
use SIU\SQ\Lib\constantes;

class paypertic extends gestor_pago
{
    /**
     * @param $id_referencia (Id externo de la cobranza)
     * @throws error
     */
    public function anular_cupon($id_referencia)
    {
        try {
            $r = lib::entorno()->get_cliente('sq_pagos_backend')->put('paypertic/expirar_payment/' . $id_referencia);
        } catch (\GuzzleHttp\Exception\ClientException $ex) {
            $msg = $ex->getResponse()->getBody()->getContents() . ' :' . $id_referencia;
            lib::entorno()->log()->err(__METHOD__ . ": " . $msg);
            throw new error($msg);
        } catch (\Exception $ex) {
            $msg = $ex->getMessage() . ' :' . $id_referencia;
            lib::entorno()->log()->err(__METHOD__ . ": " . $msg);
            throw new error($msg);
        }
    }

    /**
     * @param $id_referencia (Id externo de la cobranza)
     * @throws error
     */
    public function devolver_pago($id_referencia)
    {
        try {
            lib::entorno()->get_cliente('sq_pagos_backend')->put('paypertic/devolver_payment/' . $id_referencia);
        } catch (\GuzzleHttp\Exception\ClientException $ex) {
            $msg = $ex->getResponse()->getBody()->getContents() . ' :' . $id_referencia;
            lib::entorno()->log()->err(__METHOD__ . ": " . $msg);
            throw new error($msg);
        } catch (\Exception $ex) {
            $msg = $ex->getMessage() . ' :' . $id_referencia;
            lib::entorno()->log()->err(__METHOD__ . ": " . $msg);
            throw new error($msg);
        }
    }

    /**
     * @param $id_referencia
     * @throws error
     */
    public function expirar_url_de_pago($id_referencia)
    {
        /**
         * TODO: Expirar preferencia de PPT
         */
    }

    /**
     * @param $id_referencia
     * @return mixed
     * @throws error
     */
    public function obtener_datos_del_pago($id_referencia)
    {
        try {
            $response = lib::entorno()->get_cliente('sq_pagos_backend')->get('paypertic/payment/' . $id_referencia);
            $datos = json_decode($response->getBody()->getContents(), true);
            return $datos;
        } catch (\GuzzleHttp\Exception\ClientException $ex) {
            if ($ex->getCode() != 423) {
                $msg = $ex->getResponse()->getBody()->getContents() . ' :' . $id_referencia;
                lib::entorno()->log()->err(__METHOD__ . ": " . $msg);
                throw new error($msg);
            }
        } catch (\Exception $ex) {
            $msg = $ex->getMessage() . ' :' . $id_referencia;
            lib::entorno()->log()->err(__METHOD__ . ": " . $msg);
            throw new error($msg);
        }
    }
    
    /**
     * @param $parametros
     * @return mixed
     * @throws error
     */
    public function obtener_datos_de_pagos($parametros)
    {
        try {
            $payments = [];

            // Consulta inicial
            $response = $this->entorno()->get_cliente('sq_pagos_backend')->get('paypertic/payments', ['query' => $parametros]);

            $datos_response = json_decode($response->getBody()->getContents(), true);

            if (isset($datos_response['payments'])) {
             
                // Guardo primera tanda de payments recibidos.
                $payments = $datos_response['payments'];
                
                // Vuelvo a realizar n consultas para traer lo que falta, si es que es necesario
                if ($datos_response['size'] >= $datos_response['limit']) {
                    $cant_paginas = ceil($datos_response['size'] / $datos_response['limit']);

                    for ($pagina = 1; $pagina <= $cant_paginas; $pagina++) {
                        $parametros['page'] = $pagina;

                        $response = $this->entorno()->get_cliente('sq_pagos_backend')->get('paypertic/payments', ['query' => $parametros]);
                        $datos_response = json_decode($response->getBody()->getContents(), true);

                        // Voy sumando payments de siguientes tandas
                        $payments = array_merge($payments, $datos_response['payments']);
                    }
                }

                // devuelvo todos los payments
                return $payments;
            } else {
                return [];
            }
        } catch (\GuzzleHttp\Exception\ClientException $ex) {
            if ($ex->getCode() != 423) {
                $msg = $ex->getResponse()->getBody()->getContents();
                lib::entorno()->log()->err(__METHOD__ . ": " . $msg);
                throw new error($msg);
            }
        } catch (\Exception $ex) {
            $msg = $ex->getMessage() ;
            lib::entorno()->log()->err(__METHOD__ . ": " . $msg);
            throw new error($msg);
        }
    }

    /**
     * @throws error
     */
    public function get_metodos_pagos()
    {
    }
}
