<?php
namespace SIU\SQ\Lib\util;

use SIU\SQ\Lib\lib;

class planificador_tareas
{
    protected $entorno;
    protected $procesos;
    protected $archivo_pid;

    //TIPOS DE FRECUENCIA
    const DIA_FIJO = 'DIA FIJO';
    const DIARIO = 'DIARIO';
    const INTERVALO = 'INTERVALO';
    const CLASS_PROCESO = "SIU\SQ\Lib\procesos\proceso";

    public function __construct($archivo_pid, $entorno, $procesos)
    {
        $this->archivo_pid = $archivo_pid;
        $this->entorno = $entorno;
        $this->procesos = $procesos;
    }

    public function ejecutar_tareas_planificadas()
    {
        // evitar concurrencia
        if (file_exists($this->archivo_pid)) {
            $this->entorno->log()->notice('ERROR. Existe un proceso en ejecución');
            throw new \Exception("Existe un proceso en ejecucion", 254);
        } else {
            $pid = getmypid();
            file_put_contents($this->archivo_pid, $pid);
            if (file_exists($this->archivo_pid)) {
                $this->entorno->log()->notice('OK. Comienza la búsqueda de procesos a ejecutar');
                try {
                    $this->seleccionar_tareas_a_ejecutar();
                } catch (\Exception $ex) {
                    throw $ex;
                } finally {
                    unlink($this->archivo_pid);
                }
            } else {
                throw new \Exception("No pudo crearse el archivo sanaviron.pid. Verifique los permisos de escritura de la carpeta log.", 2);
            }
        }
    }

    public function seleccionar_tareas_a_ejecutar()
    {
        try {
            $sql = "SELECT tp.id, tp.id_proceso, tp.tipo_frecuencia, tp.dia, tp.hora, tp.intervalo,
						CASE
							WHEN pr.prioridad = 'A' THEN 1
							WHEN pr.prioridad = 'M' THEN 2
							WHEN pr.prioridad = 'B' THEN 3
						END as prioridad,
                        (SELECT COUNT(*) FROM procesos_parametrizaciones pp WHERE pp.id_proceso = pr.id) AS cantidad_parametrizaciones,
                        (SELECT COUNT(*) FROM procesos_tareas_programadas_valores ptpv WHERE ptpv.id_proceso_tarea_programada = tp.id) AS cantidad_parametrizaciones_seteadas
                        
					FROM procesos_tareas_programadas tp, procesos pr
					WHERE tp.id_proceso = pr.id
					ORDER BY prioridad ASC;";
            $tareas = $this->entorno->db()->consultar($sql);

            foreach ($tareas as $tarea) {
                $query = "SELECT id_proceso, iniciado_en, finalizado_en, id_proceso_ejecucion_estado 
                        FROM procesos_ejecuciones 
                        WHERE id_proceso = " . $tarea['id_proceso'] . "
                        ORDER BY iniciado_en DESC LIMIT 1;";
                $ultima_ejecucion = $this->entorno->db()->consultar($query);

                if ($this->cumple_condicion_ejecucion($tarea, $ultima_ejecucion)) {
                    if ($this->cumple_parametrizaciones($tarea)) {
                        if (isset($this->procesos[$tarea['id_proceso']]) && is_object($this->procesos[$tarea['id_proceso']])
                            && is_subclass_of($this->procesos[$tarea['id_proceso']], self::CLASS_PROCESO)) {
                            if ($tarea['cantidad_parametrizaciones'] != 0) {
                                $this->set_parametros_a_proceso($tarea);
                            }
                            $this->procesos[$tarea['id_proceso']]->ejecutar();
                        } else {
                            $msg = 'PLANIFICADOR DE TAREAS. El proceso con id ' . $tarea['id_proceso'] . ' no se encuentra configurado en el comando "tareas_planificadas"';
                            $this->entorno->log()->addError($msg);
                            throw new \Exception($msg);
                        }
                    } else {
                        $msg = 'PLANIFICADOR DE TAREAS. El proceso con id ' . $tarea['id_proceso'] . ' no posee configurados sus parámetros de sistema en operación: "planificador de tareas"';
                        $this->entorno->log()->addError($msg);
                    }
                }
            }
        } catch (\SIU\SQ\Lib\errores\error $e) {
            throw $e;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function cumple_parametrizaciones($tarea)
    {
        // Verificar cantidad de parametros que requiere el proceso y la cantidad configurada para esa tarea.
        return ($tarea['cantidad_parametrizaciones_seteadas'] == $tarea['cantidad_parametrizaciones']);
    }

    public function set_parametros_a_proceso($tarea)
    {
        // Llego hasta aquí confiado que todos los parámetros fueron seteados.
        $valores = lib::catalogo_daos()->procesos()->get_tareas_programadas_valores(['id_proceso_tarea_programada' => $tarea['id']]);

        foreach ($valores as $valor) {
            $this->procesos[$tarea['id_proceso']]->set_parametro($valor['id_proceso_parametro'], $valor['valor']);
        }
    }

    public function cumple_condicion_ejecucion($tarea, $ultima_ejecucion)
    {
        $ahora = date_create_from_format('Y-m-d H:i:s.u', date('Y-m-d H:i:s.u'));
        $es_el_dia = $ahora->format('w');
        $hora_a_ejecutar = date_create_from_format('H:i', $tarea['hora']);
        $dia_a_ejecutar = $tarea['dia'];
        if (!empty($ultima_ejecucion)) {
            if (strlen($ultima_ejecucion[0]['iniciado_en']) > 21) {
                $hora_ultima_ejecucion = date_create_from_format('Y-m-d H:i:s.u', $ultima_ejecucion[0]['iniciado_en']);
            }else{
                $hora_ultima_ejecucion = date_create_from_format('Y-m-d H:i:s', $ultima_ejecucion[0]['iniciado_en']);
            }
            switch ($tarea['tipo_frecuencia']) {
                case self::DIA_FIJO:
                    return $hora_a_ejecutar > $hora_ultima_ejecucion && $es_el_dia == $dia_a_ejecutar && $ahora >= $hora_a_ejecutar;
                case self::DIARIO:
                    return $hora_a_ejecutar > $hora_ultima_ejecucion && $ahora >= $hora_a_ejecutar;
                case self::INTERVALO:
                    $intervalo = new \DateInterval('PT' . $tarea['intervalo'] . 'M');
                    return $ahora >= $hora_ultima_ejecucion->add($intervalo);
                default:
                    break;
            }
        } else {
            switch ($tarea['tipo_frecuencia']) {
                case self::DIA_FIJO:
                    return $es_el_dia == $dia_a_ejecutar && $ahora >= $hora_a_ejecutar;
                case self::DIARIO:
                    return $ahora >= $hora_a_ejecutar;
                case self::INTERVALO:
                    return true;
                default:
                    break;
            }
        }
    }
}
